@@ -1,4 +1,4 @@
-"""honeybee-vtk command line interface."""
+"""honeybee-vtk command to translate HBJSON to different formats."""
 
 import sys
 import pathlib
@@ -12,7 +12,7 @@ from honeybee_vtk.types import VTKWriters
 
 @click.group()
 def translate():
-    """Honeybee VTK commands entry point."""
+    """Command to translate a honeybee-vtk model into other formats."""
     pass
 
 
