@@ -1,13 +1,11 @@
 """A VTK representation of HBModel."""
 
 from __future__ import annotations
-from json.decoder import JSONDecodeError
 import pathlib
 import shutil
 import webbrowser
 import tempfile
 import os
-import tempfile
 import json
 import warnings
 
@@ -15,9 +13,10 @@ from collections import defaultdict
 from typing import Dict, List, Union, Tuple
 
 from honeybee.facetype import face_types
+from honeybee.typing import clean_string
 from honeybee.model import Model as HBModel
 from honeybee_radiance.sensorgrid import SensorGrid
-from honeybee_radiance.sensor import Sensor
+from honeybee_radiance.writer import _filter_by_pattern
 from ladybug.color import Color
 from ladybug_geometry.geometry3d import Mesh3D
 
@@ -30,7 +29,7 @@ from .to_vtk import convert_aperture, convert_face, convert_room, convert_shade,
 from .vtkjs.schema import IndexJSON, DisplayMode, SensorGridOptions
 from .vtkjs.helper import convert_directory_to_zip_file, add_data_to_viewer
 from .types import DataSetNames, VTKWriters, JoinedPolyData, ImageTypes
-from .config import DataConfig, Autocalculate, Config
+from .config import DataConfig, Autocalculate
 from .legend_parameter import Text
 from .text_actor import TextActor
 
@@ -105,7 +104,9 @@ class Model(object):
 
     @classmethod
     def from_hbjson(cls, hbjson: str,
-                    load_grids: SensorGridOptions = SensorGridOptions.Ignore) -> Model:
+                    load_grids: SensorGridOptions = SensorGridOptions.Ignore,
+                    grids_filter: Union[str, List[str]] = '*',
+                    full_match: bool = False) -> Model:
         """Translate hbjson to a honeybee-vtk model.
 
         Args:
@@ -202,40 +203,56 @@ class Model(object):
         """Load sensor grids."""
         if self._sensor_grids_option == SensorGridOptions.Ignore:
             return
-        if hasattr(self._hb_model.properties, 'radiance'):
+        if hasattr(self._hb_model.properties, 'radiance') and \
+                self._hb_model.properties.radiance.sensor_grids:
+
+            grids = self._hb_model.properties.radiance.sensor_grids
+
             # list of unique sensor_grid identifiers in the model
-            ids = set([grid.identifier for grid in
-                       self._hb_model.properties.radiance.sensor_grids])
+            ids = set([grid.identifier for grid in grids])
 
             # if all the grids have the same identifier, merge them into one grid
             if len(ids) == 1:
-                id = self._hb_model.properties.radiance.sensor_grids[0].identifier
+                id = grids[0].identifier
 
                 # if it's just one grid, use it
-                if len(self._hb_model.properties.radiance.sensor_grids) == 1:
-                    sensor_grid = self._hb_model.properties.radiance.sensor_grids[0]
+                if len(grids) == 1:
+                    sensor_grid = grids[0]
                 # if there are more than one grid, merge them first
                 else:
-                    grid_meshes = [grid.mesh for grid in
-                                   self._hb_model.properties.radiance.sensor_grids]
+                    grid_meshes = [grid.mesh for grid in grids]
                     if all(grid_meshes):
                         mesh = Mesh3D.join_meshes(grid_meshes)
                         sensor_grid = SensorGrid.from_mesh3d(id, mesh)
                     else:
-                        sensors = [sensor for grid in
-                                   self._hb_model.properties.radiance.sensor_grids
-                                   for sensor in grid.sensors]
+                        sensors = [sensor for grid in grids for sensor
+                                   in grid.sensors]
                         sensor_grid = SensorGrid(id, sensors)
 
-                self._sensor_grids.data.append(
+                # TODO extract this to a function
+                try:
                     convert_sensor_grid(sensor_grid, self._sensor_grids_option)
-                )
+                except ValueError:
+                    warnings.warn(f'Grid {id} does not have mesh information. Hence, '
+                                  'it will not be converted to a sensor grid. Try with'
+                                  ' SensorGridOptions.Sensors.')
+                else:
+                    self._sensor_grids.data.append(convert_sensor_grid(
+                        sensor_grid, self._sensor_grids_option))
             # else add them as separate grids
             else:
-                for sensor_grid in self._hb_model.properties.radiance.sensor_grids:
-                    self._sensor_grids.data.append(
+                for sensor_grid in grids:
+                    # TODO extract this to a function
+                    try:
                         convert_sensor_grid(sensor_grid, self._sensor_grids_option)
-                    )
+                    except ValueError:
+                        warnings.warn(f'Grid {sensor_grid.identifier} does not have'
+                                      ' mesh information. Hence, it will not be'
+                                      ' converted to a sensor grid. Try with'
+                                      ' SensorGridOptions.Sensors.')
+                    else:
+                        self._sensor_grids.data.append(convert_sensor_grid(
+                            sensor_grid, self._sensor_grids_option))
 
     def _load_cameras(self) -> None:
         """Load radiance views."""
@@ -573,7 +590,6 @@ class Model(object):
 
         for actor in scene.actors:
             if actor.name == 'Grid':
-                print('Grid display mode', actor.modeldataset.display_mode)
                 assert actor.modeldataset.display_mode == grid_display_mode, 'Grid display'\
                     ' mode is not set correctly.'
             else:
@@ -585,7 +601,8 @@ class Model(object):
             image_width=image_width, image_height=image_height)
 
     def to_grid_images(self, config: str, *, folder: str = '.',
-                       grid_filter: List[str] = None,
+                       grids_filter: Union[str, List[str]] = '*',
+                       full_match: bool = False,
                        grid_display_mode: DisplayMode = DisplayMode.SurfaceWithEdges,
                        background_color: Tuple[int, int, int] = None,
                        image_type: ImageTypes = ImageTypes.png,
@@ -593,7 +610,7 @@ class Model(object):
                        text_actor: TextActor = None,
                        grid_camera_dict: Dict[str, vtk.vtkCamera] = None,
                        extract_camera: bool = False,
-                       grid_color: Color = None,
+                       grid_colors: List[Color] = None,
                        sub_folder_name: str = None) -> Union[Dict[str, Camera],
                                                              List[str]]:
         """Export am image for each grid in the model.
@@ -613,8 +630,10 @@ class Model(object):
             config: Path to the config file in JSON format.
             folder: Path to the folder where you'd like to export the images. Defaults to
                     the current working directory.
-            grid_filter: A list of grid identifiers to export images of those grid only.
-                Defaults to None which will export all the grids.
+            grids_filter: A list of grid identifiers or a regex pattern as a string to
+                filter the grids. Defaults to None.
+            full_match: A boolean to filter grids by their identifiers as full matches.
+                Defaults to False.
             display_mode: Display mode for the grid. Defaults to surface with edges.
             background_color: Background color of the image. Defaults to white.
             image_type: Image type to be exported. Defaults to png.
@@ -631,12 +650,12 @@ class Model(object):
                 to be used in this run to export an image. Defaults to None.
             extract_camera: Boolean to indicate whether to extract the camera from the
                 for this run to use for the next run. Defaults to False.
-            grid_color: Color of the grid. This is used to set the color of the grid
-                when exporting the time step images when the whole grid needs to be
-                of a single color. Defaults to None.
+            grid_colors: Colors used to color the grid. Provide only one color in the
+                list to color the grid with the same color. This is useful in exporting
+                time step images. Defaults to None.
             sub_folder_name: A text string that sets the name of the subfolder where
                 the images will be exported. This is useful when the images are to be
-                exported for multiple time periods such as whole day of June 21, and 
+                exported for multiple time periods such as whole day of June 21, and
                 the whole day of March, 21. Defaults to None.
 
         Returns:
@@ -649,7 +668,9 @@ class Model(object):
             grid_display_mode = DisplayMode.Points
 
         config_data = self.load_config(config)
-        grid_polydata_lst = filter_grids(self.sensor_grids.data, grid_filter)
+
+        grid_polydata_lst = _filter_grid_polydata(
+            self.sensor_grids.data, self._hb_model, grids_filter, full_match)
 
         output: Union[Dict[str, Camera], List[str]] = {} if extract_camera else []
 
@@ -658,9 +679,14 @@ class Model(object):
                 dataset = ModelDataSet(name=grid_polydata.identifier,
                                        data=[grid_polydata],
                                        display_mode=grid_display_mode)
-                if grid_color:
-                    dataset.active_field_info.legend_parameter._assign_colors(
-                        (Color(255, 255, 255), grid_color))
+                if grid_colors:
+                    if len(grid_colors) == 1:
+                        dataset.active_field_info.legend_parameter._assign_colors(
+                            [Color(255, 255, 255), grid_colors[0]])
+                    else:
+                        dataset.active_field_info.legend_parameter._assign_colors(
+                            grid_colors)
+
                 dataset.color_by = data.identifier
                 actor = Actor(dataset)
                 camera = _camera_to_grid_actor(actor, data.identifier)
@@ -690,9 +716,8 @@ class Model(object):
                         grid_folder = pathlib.Path(
                             f'{folder}/{grid_polydata.identifier}/{sub_folder_name}')
 
-                    if grid_folder.exists():
-                        shutil.rmtree(grid_folder)
-                    grid_folder.mkdir(parents=True, exist_ok=True)
+                    if not grid_folder.exists():
+                        grid_folder.mkdir(parents=True, exist_ok=True)
 
                     output += scene.export_images(folder=grid_folder,
                                                   image_type=image_type,
@@ -801,8 +826,8 @@ class Model(object):
         """Match result data with the sensor grids in the model.
 
         It will be checked if the number of data files and the names of the
-        data files match with the grid identifiers. This function does not support validating
-        result data for other than sensor grids as of now.
+        data files match with the grid identifiers. This function does not support
+        validating result data for other than sensor grids as of now.
 
         This is a helper method to the public load_config method.
 
@@ -825,7 +850,7 @@ class Model(object):
 
         # match identifiers of the grids with the identifiers of the result files
         grids_model_identifiers = [grid.identifier for grid in self.sensor_grids.data]
-        grids_info_identifiers = [grid['full_id'] for grid in grids_info]
+        grids_info_identifiers = [grid['identifier'] for grid in grids_info]
         assert grids_model_identifiers == grids_info_identifiers, 'The identifiers of'\
             ' the sensor grids in the model do not match the identifiers of the grids'\
             f' in the grids_info.json for {data.identifier}.'
@@ -843,7 +868,8 @@ class Model(object):
             num_sensors = [polydata.GetNumberOfCells()
                            for polydata in self.sensor_grids.data]
 
-        # lastly check if the length of a file matches the number of sensors or meshes on grid
+        # lastly check if the length of a file matches the number of sensors or
+        # meshes on grid
         if file_lengths != num_sensors:
             length_matching = {
                 grids_info_identifiers[i]: file_lengths[i] == num_sensors[i] for i in
@@ -865,12 +891,15 @@ class Model(object):
         Args:
             data (DataConfig): A Dataconfig object.
             model: A honeybee-vtk model.
-            grid_type: A string indicating whether the sensor grid in the model is made of
-                points or meshes.
-            legend_range: A list of min and max values of the legend parameters provided by
-                the user in the config file.
+            grid_type: A string indicating whether the sensor grid in the model is
+                made of points or meshes.
+            legend_range: A list of min and max values of the legend parameters
+                provided by the user in the config file.
         """
         folder_path = pathlib.Path(data.path)
+        folder_path = folder_path.as_posix()
+        folder_path = pathlib.Path(folder_path)
+
         identifier = data.identifier
         if isinstance(data.lower_threshold, float):
             lower_threshold = data.lower_threshold
@@ -881,28 +910,12 @@ class Model(object):
         else:
             upper_threshold = None
 
-        grids_info_json = folder_path.joinpath('grids_info.json')
-        with open(grids_info_json) as fh:
-            grids_info = json.load(fh)
-
-        # grid identifier from grids_info.json
-        file_names = [grid['identifier'] for grid in grids_info]
-
-        # finding file extension for grid results
-        for path in folder_path.iterdir():
-            if path.stem == file_names[0]:
-                extension = path.suffix
-                break
-
-        # file paths to the result files
-        file_paths = [folder_path.joinpath(name+extension)
-                      for name in file_names]
-
+        result_file_paths = _get_result_file_paths(folder_path)
         result = []
-        for file_path in file_paths:
-            res_file = pathlib.Path(file_path)
+        for res_file_path in result_file_paths:
+            assert res_file_path.exists(), f'No file found at {res_file_path}'
             grid_res = [float(v)
-                        for v in res_file.read_text().splitlines()]
+                        for v in res_file_path.read_text().splitlines()]
             result.append(grid_res)
 
         ds = self.get_modeldataset(DataSetNames.grid)
@@ -960,6 +973,8 @@ class Model(object):
         legend_params = data.legend_parameters
         legend = scene.legend_parameter(data.identifier)
         legend.colorset = legend_params.color_set
+        legend.reverse_colorset = legend_params.reverse_color_set
+
         legend.unit = data.unit
         if legend_range:
             legend.min, legend.max = legend_range
@@ -1017,7 +1032,6 @@ def _camera_to_grid_actor(actor: Actor, data_name: str, zoom: int = 2,
     Returns:
         A Camera object.
     """
-
     cent_pt = actor.centroid
     return Camera(identifier=f'{data_name}_{actor.name}',
                   position=(cent_pt.x, cent_pt.y, cent_pt.z + camera_offset),
@@ -1028,23 +1042,54 @@ def _camera_to_grid_actor(actor: Actor, data_name: str, zoom: int = 2,
                   reset_camera=auto_zoom)
 
 
-def filter_grids(grid_polydata_lst: List[PolyData],
-                 grid_filter: List[str]) -> List[PolyData]:
-    """Filter grids based on the grid_filter.
+def _get_result_file_paths(folder_path: Union[str, pathlib.Path]):
+
+    if not isinstance(folder_path, pathlib.Path):
+        folder_path = pathlib.Path(folder_path)
+
+    grids_info_json = folder_path.joinpath('grids_info.json')
+    with open(grids_info_json) as fh:
+        grids_info = json.load(fh)
+
+    # finding file extension for grid results
+    # This could have been avoided if the file extension was provided in the
+    # grids_info.json file.
+    extension = None
+    for path in folder_path.rglob('*'):
+        if path.stem == grids_info[0]['identifier']:
+            extension = path.suffix
+            break
+
+    # result file paths
+    return [folder_path.joinpath(f"{grid['full_id']}{extension}")
+            for grid in grids_info]
+
+
+def _filter_grid_polydata(grid_polydata_lst: List[PolyData], model: HBModel,
+                          grids_filter: Union[str, List[str]],
+                          full_match) -> List[PolyData]:
+    """Filter grid polydata based on sensor grids.
 
     Args:
-        grid_polydata_lst: A list of PolyData objects for Grids.
-        grid_filter: A list of grid identifiers to filter grids.
+        grid_polydata_lst: A list of grid polydata objects.
+        model: A honeybee model object.
+        grids_filter: A list of grid identifiers or a regex pattern as a string to filter
+            the grid polydata.
+        full_match: A boolean to filter grids by their identifiers as full matches.
 
     Returns:
         A list of PolyData objects for Grids.
     """
-    if not grid_filter:
+    if not grids_filter or grids_filter == '*':
         return grid_polydata_lst
     else:
+        filtered_sensor_grids = _filter_by_pattern(
+            model.properties.radiance.sensor_grids, grids_filter, full_match)
+        sensorgrid_identifiers = [
+            grid.identifier for grid in filtered_sensor_grids]
         filtered_grid_polydata_lst = [grid for grid in grid_polydata_lst
-                                      if grid.name in grid_filter]
+                                      if grid.name in sensorgrid_identifiers]
         if not filtered_grid_polydata_lst:
-            raise ValueError('No grids found in the model that match the filter'
-                             ' defined in the config file.')
+            raise ValueError('No grids found in the model that match the'
+                             f' filter {grids_filter}.')
         return filtered_grid_polydata_lst
